%
% This script will animate the belt system
% Written by SAB, on June 11, 1998
%

%
% First, define the geometry of the pulley system.  First, thea fixed,
% time-invariant terms are defined.  These do not depend on q1, q2
% or q3.  THESE ARE THE ONLY VARIABLES THAT CAN BE CHANGED TO AFFECT
% THE APPEARANCE OF THE ANIMATION - EVERYTHING ELSE IS COMPUTED FROM
% THESE AND / OR q1, q2, or q3
%

xc1=50;  % x-coordinate of center of pulley 1
yc1=50;  % y-coordinate of center of pulley 1
r1=25;   % radius of pulley 1
r2=10;   % radius of pulley 2
d1=100;  % x-axis distance from center of pulley 1 to dancer
d3=5;    % nominal height of dancer pulleys
r3=5;    % radius of dancer pulleys
d4=3;    % radius of link 1
d5=2;    % radius of dancer pulley holder
d6=60;   % length of link 2
d7=6;    % radius of link 2

% Now compute the coordinates of pulley 2...

temp=sqrt(d1^2+d3^2-r1^2);
d2=(d1*d3^2 + d3*r1*temp - d1*r1*r2 - d3*temp*r2)/(d3^2 - r1^2)
xc2=xc1+d1+d2; % x-coordinate of center of pulley 2
yc2=yc1;       % y-coordinate of center of pulley 2

% This is used to make all circles...

theta=0:0.1:2*pi;

% This chooses figure 1.  Other figures can be used to make plots.

figptr=figure(3);

skip=floor(length(x)/100);

M = moviein(floor(length(x)/skip)); % allocate memory for the movie

for j=1:floor(length(x)/skip),      % this is the length of the movie

%
% Some values of qi for testing purposes...
%

q1=x(skip*j,1);
q2=x(skip*j,2);
q3=100*x(skip*j,3); % x100 since the animation dimensions are 10x the sim dimension.

%q1=0.5*(50-2*j)/50*d3;
%q2=(50-2*j)/50*d3;
%q3=(50-2*j)/100*d3;

% Clear the graph of each frame, and set up the axes..

clg;
axis([xc1-d6 xc2+r2 yc1-d6 yc1+d6]);  % this has to be set manually
% set(gca,'aspectratio',[1 1]);  % this makes the picture small within the frame
set(gca,'clipping','off');
set(gca,'visible','off');

%
% Locations of dancer pulleys...
%

xc3=xc1+d1;
yc3=yc1+d3+r3+q3;

xc4=xc1+d1;
yc4=yc1-d3-r3+q3;

% Draw the link 1 rectangle...
patch([xc1-d4 xc1+d1+d2+d4 xc1+d1+d2+d4 xc1-d4],[yc1-d4 yc1-d4 yc1+d4 yc1+d4],'b');

% Draw the link two triangle
patch([xc1-d7*sin(q1) xc1-d6*cos(q1) xc1+d7*sin(q1)],...
[yc1+d7*cos(q1) yc1-d6*sin(q1) yc1-d7*cos(q1)],'b');

% Locations of the four belt pieces...

% length of top of belt, left and then right...
l1=(d1^2+(d3+q3)^2-r1^2)^.5;
l2=(d2^2+(d3+q3)^2-r2^2)^.5;

% length of bottom of belt, first right and then left...
l3=(d2^2+(d3-q3)^2-r2^2)^.5;
l4=(d1^2+(d3-q3)^2-r1^2)^.5;

% left pulley angles, with alpha1 the top, and beta1 the bottom...
alpha1=atan((d3+q3)/d1)+atan(l1/r1);
beta1=atan((d3-q3)/d1)+atan(l4/r1);

% right pulley angles, with alpha2 the top, and beta2 the bottom...
alpha2=atan((d3+q3)/d2)+atan(l2/r2);
beta2=atan((d3-q3)/d2)+atan(l3/r2);

% draw the four belt pieces...
line([xc1+r1*cos(alpha1) xc1+d1], [yc1+r1*sin(alpha1) yc1+d3+q3],'color','g','linewidth',3);
line([xc1+r1*cos(beta1) xc1+d1], [yc1-r1*sin(beta1) yc1-d3+q3],'color','g','linewidth',3);

line([xc2-r2*cos(alpha2) xc1+d1], [yc2+r2*sin(alpha2) yc1+d3+q3],'color','g','linewidth',3);
line([xc2-r2*cos(beta2) xc1+d1], [yc2-r2*sin(beta2) yc1-d3+q3],'color','g','linewidth',3);

% and the two belt arcs...
theta1=alpha1:0.1:2*pi-beta1;        % range of angles around pulley 1 where belt is in contact;
theta2=-pi+beta2:0.1:pi-alpha2;      % range of angles around pulley 2 where belt is in contact;

xleft=xc1+r1*cos(theta1);
yleft=yc1+r1*sin(theta1);
line(xleft,yleft,'color','g','linewidth',3);

xright=xc2+r2*cos(theta2);
yright=yc2+r2*sin(theta2);
line(xright,yright,'color','g','linewidth',3);

%
% Next we draw the dancer pulley holder rectangle...
%

patch([xc1+d1-d5 xc1+d1+d5 xc1+d1+d5 xc1+d1-d5],...
      [yc1-d3+q3-r3 yc1-d3+q3-r3 yc1+d3+q3+r3 yc1+d3+q3+r3],'c'); 

%
% Next let's draw the two DANCER pulleys...
%

x3=xc3+r3*sin(theta);
y3=yc3+r3*cos(theta);
patch(x3,y3,'r');

x4=xc4+r3*sin(theta);
y4=yc4+r3*cos(theta);
patch(x4,y4,'r');

%
% And then draw the two FIXED pulleys...
%

x1=xc1+r1*sin(theta);   % drawing the first pulley circle now.
y1=yc1+r1*cos(theta);
patch(x1,y1,'r');

line([xc1+r1*cos(q1) xc1+r1*cos(q1+pi)],...
     [yc1+r1*sin(q1) yc1+r1*sin(q1+pi)],'color','y');
line([xc1+r1*cos(q1+pi/4) xc1+r1*cos(q1+5*pi/4)],...
     [yc1+r1*sin(q1+pi/4) yc1+r1*sin(q1+5*pi/4)],'color','y');
line([xc1+r1*cos(q1+pi/2) xc1+r1*cos(q1+3*pi/2)],...
     [yc1+r1*sin(q1+pi/2) yc1+r1*sin(q1+3*pi/2)],'color','y');
line([xc1+r1*cos(q1+3*pi/4) xc1+r1*cos(q1+7*pi/4)],...
     [yc1+r1*sin(q1+3*pi/4) yc1+r1*sin(q1+7*pi/4)],'color','y');

x2=xc2+r2*sin(theta);  % drawing the second pulley circle now.
y2=yc2+r2*cos(theta);
patch(x2,y2,'r');

line([xc2+r2*cos(q2) xc2+r2*cos(q2+pi)],...
     [yc2+r2*sin(q2) yc2+r2*sin(q2+pi)],'color','y');
line([xc2+r2*cos(q2+pi/4) xc2+r2*cos(q2+5*pi/4)],...
     [yc2+r2*sin(q2+pi/4) yc2+r2*sin(q2+5*pi/4)],'color','y');
line([xc2+r2*cos(q2+pi/2) xc2+r2*cos(q2+3*pi/2)],...
     [yc2+r2*sin(q2+pi/2) yc2+r2*sin(q2+3*pi/2)],'color','y');
line([xc2+r2*cos(q2+3*pi/4) xc2+r2*cos(q2+7*pi/4)],...
     [yc2+r2*sin(q2+3*pi/4) yc2+r2*sin(q2+7*pi/4)],'color','y');

% And finally, some text...
% text(1.5*xc1,yc1+1.5*r1,'Pulley Animation','fontsize',36)

textspace=12;

text(xc1-0.5*r1,yc1-1.5*r1,'Output','fontsize',24,'fontweight','bold')
text(xc1-0.5*r1,yc1-1.5*r1-textspace,'Pulley','fontsize',24,'fontweight','bold')

text(xc2-r2,yc2-2.0*r2,'Actuated','fontsize',24,'fontweight','bold')
text(xc2-r2,yc2-2.0*r2-textspace,'Pulley','fontsize',24,'fontweight','bold')

text(xc1,yc1+2.0*r1,'u1=PD, u2=0, K=1000','fontsize',24,'fontweight','bold')

M(:,j) = getframe;

end;
